@@ -1,13 +1,18 @@
+/*
+ * Public domain
+ * unistd.h compatibility shim
+ */
+
 #include_next <unistd.h>
 
 #ifndef LIBCRYPTOCOMPAT_UNISTD_H
 #define LIBCRYPTOCOMPAT_UNISTD_H
 
-#ifdef NO_GETENTROPY
+#ifndef HAVE_GETENTROPY
 int getentropy(void *buf, size_t buflen);
 #endif
 
-#ifdef NO_ISSETUGID
+#ifndef HAVE_ISSETUGID
 int issetugid(void);
 #endif
 
